// *************************************************************************
// * Program		: diint.c
// * Description	: Demo program for digital input with interrupt function
// *
// * Revision		: 1.0
// * Date		: 08/24/2008  Advantech Co., Ltd.
// *************************************************************************
#include <sys/time.h>
#include <sys/types.h>
#include "../include/unodio.h"

struct timespec g_init_budget = { 5, 0 };
struct timespec g_repl_period = { 10, 0 };

#define MY_REPL_PERIOD g_repl_period
#define MY_INIT_BUDGET g_init_budget
#define MY_MAX_REPL 10

static pthread_t han_in_id;

static int di0int = 0;
static int di1int = 0;

void * handle_interrupt(void * arg)
{
   int event_type;
   struct timeval tvDI0Old, tvDI0New, tvDI1Old, tvDI1New;
   gettimeofday( &tvDI0Old, NULL );
   gettimeofday( &tvDI1Old, NULL );
   while(1)
   {
      event_type = CheckEvent( 1 );
      switch( event_type ) 
      {
         case EVT_DI0:
            di0int ++;
            gettimeofday(&tvDI0New,NULL);
            if (tvDI0New.tv_sec-tvDI0Old.tv_sec >= 1)
            {
               printf("DI0 interrupt ocurred %.3f/s\n",
                  (float)di0int/(tvDI0New.tv_sec-tvDI0Old.tv_sec));
               tvDI0Old = tvDI0New;
               di0int = 0;
            }
         break;
         case EVT_DI1:
            di1int ++;
            gettimeofday(&tvDI1New,NULL);
            if (tvDI1New.tv_sec-tvDI1Old.tv_sec >= 1)
            {
               printf("DI1 interrupt ocurred %.3f/s\n",
                  (float)di1int/(tvDI1New.tv_sec-tvDI1Old.tv_sec));
               tvDI1Old = tvDI1New;
               di1int = 0;
            }
         break;
         case EVT_TIMEOUT:
            printf("time out!\n");
            break;
      }
   }
}

int main ( void )
{ 
   int EventType = 0;
   int ret;   
   //Step 1: Display hardware and software settings
   printf("------ DI interrupt function testing ------\n\n");
   
   //Step 2: Open device
   ret = DeviceOpen();  
   if (ret < 0 )
   {
      printf("Device Open error!\n");
      return 1;
   }
    	
   // Step3: configure the DI interrupt for rising edge
   EventType = ADS_EVT_DI0 | ADS_EVT_DI1;	
   
   if ((EventConfig(EventType, EVENT_EDGE_RISING) == -1) && errno == ENOTSUP)
   {
      printf("This funtion do not support\n");
      return 1; 
   }
 

   // Step4: enable and start the event function
   EventType = ADS_EVT_DI0 | ADS_EVT_DI1;
   if (EventEnable(EventType, 1, 1))
      return 1;

   // Step5: exit while pressing any key
   printf("Input signal into DI0 or DI1, let the system generates interrupt!\n");
   printf("Press 'q' to exit.\n");
   ret = pthread_create( &han_in_id, NULL, handle_interrupt, NULL ); 	
   if ( ret != EOK )
   {
      printf("create thread failure!\n");
      perror( "pthread_create interrrupt:" );    
      return EXIT_FAILURE;
   } 
   while ( 1 )
   { 
      if ( getchar() == 'q' )
      {   
         pthread_cancel(han_in_id);			
         break;
      } 
      delay( 1000 );
   }      	
        
   // Step6: disable and close the event function
   EventType = ADS_EVT_DI0 | ADS_EVT_DI1;
   if (EventEnable(EventType, 0, 1))
      return 1;

   // Step7: close device handle
   DeviceClose();	
   return 0;
}
