// ***********************************************************************
// * Program        : freq.c
// * Description    : Demo program for frequence measure function
// *
// * Revision       : 1.0
// * Date           : 08/24/2008          Advantech Co., Ltd.
// ***********************************************************************
#include "../include/unodio.h"
#define SUCCESS 0

int main ( void )
{
   int ret;
   int usChan;//, nKey;	
   float  fFreq = 0.0; 	
   int retval;
   fd_set rfds;
   struct timeval tv;
   char buf[128];

   printf("\n------ Frequency measure function testing ------\n");

   //Step 1: Open device
   ret = DeviceOpen();
   if (ret == -1)
   {
      printf("Device Open error!\n");
      return 1;
   }

   // Step 2: input channel, only support 0 or 1
   printf("\nPlease input parameters ---> counter channel:\n");
   scanf("%d", &usChan);
   if ( ( usChan < 0 ) || ( usChan > 1 ) )
   {
      printf("Channel invalid!\n");
      return 1;
   }

   // Step 3: config clock pin, gate pin and out pin direction
   // ClockPin= CFG_CLOCK_INTERNAL or CFG_CLOCK_EXTERNAL
   // GatePin = CFG_GATE_INTERNAL  or CFG_GATE_EXTERNAL
   // OutPin  = CFG_OUTPIN_TO_DO   or CFG_OUTPIN_NON_DO
   ret = CounterConfig(usChan,
                 CFG_CLOCK_INTERNAL/*CFG_CLOCK_EXTERNAL*/, 
                 CFG_GATE_INTERNAL, 
                 CFG_OUTPIN_TO_DO);	
   if (ret == -1 && errno == ENOTSUP)
   {
      printf("This funtion do not support\n");
      return 1; 
   }

   ret = CounterClockConfig(TIME_BASE_1K);
   if ( ret != 0 )
      return 1;
		    
   // Step 4: Start frequency measurement operation by CounterFreqStart    
   //----CTRGate  = CFG_CTR_START or CFG_CTR_STOP; // internal control to start or stop
   printf("\nfrequency measurement starts ....(GatePeriod = 0, CTRGate = CFG_CTR_START) ");
   ret = CounterFreqStart(usChan,  CFG_CTR_START);
   if ( ret != SUCCESS )
      return 1;
    
   sleep(5);  // Waiting sometime start to read freq's value

   // Step 5: Read counter value by CounterFreqRead in while loop
   // and display counter value, exit when pressing any key

   while ( 1 )
   {
      ret = CounterFreqRead( usChan, &fFreq );
      if ( ret != 0) 
      {
         return 0;
      }

      printf("\nFrequency = %fHz, press 'q' to exit.", fFreq);

      FD_ZERO( &rfds );
      FD_SET( 0, &rfds );
      // Wait up to one second 
      tv.tv_sec = 5;
      tv.tv_usec = 0;
      retval = select( 1, &rfds, NULL, NULL, &tv);
      // Don't rely on the value of tv now! 
      if (retval == -1)
         perror("select()");
      else if ( retval )
      {
         // FD_ISSET(0, &rfds) will be true. 
         if ( fgets( buf, sizeof(buf), stdin ) == NULL )
            perror("fgets()");
         else if ( buf[ 0 ] == 'q' )
         {
            printf("\n");
            break;
         }
      }
   }

   // Step 6: Stop counter by CounterReset
   ret = CounterReset( usChan );
   if ( ret != SUCCESS )	
   {
      printf("CounterReset error!\n");
      return 1;
   }

   // Step 7: Close device
   DeviceClose();

   return 0;		
}
