// *************************************************************
// * Program        : pulse.c
// * Description    : Demo program for pulse output function
// *
// * Revision       : 1.0
// * Date           : 08/24/2008          Advantech Co., Ltd.
// **************************************************************

#include "../include/unodio.h"
#define SUCCESS 0

int main ( void )
{
   int dwErrCde;
   int usChan;
   int nTimeBase, nTimeBaseHW = TIME_BASE_100;

   //Step 1: Display hardware and software settings for running this example
   printf("\n------ pulse output function test example ------\n");

   //Step 2: Open device
   dwErrCde = DeviceOpen();
   if ( dwErrCde == -1 )
   {
      printf("Device Open error!\n");
      return 1;
   }

   // Step 3: input channel number, that only support 0 or 1
   printf("\nPlease input the counter channel:\n");
   scanf("%d", &usChan);

   // Step 4: Config counter (items: time base chosing)
   printf("\nPlease select the time base mode: ");
   printf("\n0 -- TIME BASE is 100K ");
   printf("\n1 -- TIME BASE is 10K  ");
   printf("\n2 -- TIME BASE is 1K   ");
   printf("\n3 -- TIME BASE is 100  \n");
   scanf("%d", &nTimeBase); // make sure the input is from 0 to 3
   if (nTimeBase == 0) 
      nTimeBaseHW = TIME_BASE_100K;
   else if (nTimeBase == 1) 
      nTimeBaseHW = TIME_BASE_10K;
   else if (nTimeBase == 2) 
      nTimeBaseHW = TIME_BASE_1K;
   else if (nTimeBase == 3)  
      nTimeBaseHW = TIME_BASE_100;

   dwErrCde = CounterClockConfig(nTimeBaseHW);	
   if (dwErrCde == -1 && errno == ENOTSUP)
   {
      printf("This funtion do not support\n");
      return 1; 
   }
    
   // Step 5: config clock pin, gate pin and out pin direction
   // ClockPin= CFG_CLOCK_INTERNAL or CFG_CLOCK_EXTERNAL
   // GatePin = CFG_GATE_INTERNAL  or CFG_GATE_EXTERNAL
   // OutPin  = CFG_OUTPIN_TO_DO   or CFG_OUTPIN_NON_DO
   dwErrCde = CounterConfig(usChan, 
                 CFG_CLOCK_INTERNAL, 
                 CFG_GATE_INTERNAL, 
                 CFG_OUTPIN_TO_DO);	
   if ( dwErrCde != SUCCESS )
      return 1;
   		
   // Step 6: Start counter operation by CounterPulseStart    
   //----period   = (float)0.1ms; 
   //----CTRGate  = CFG_CTR_START or CFG_CTR_STOP; // internal control to start or stop
   printf("\nPulse output starts . (Period= 100 ms)...");

   dwErrCde = CounterPulseStart(usChan, 
                 (float)100.0f, 
                 CFG_CTR_START);
   if ( dwErrCde != SUCCESS )
      return 1;

   // Step 7: exit while pressing any key
   printf("\nPress any key to exit....\n");
   getchar();
   getchar();

   // Step 8: Stop pulse output by CounterReset
   dwErrCde = CounterReset( usChan );

   // Step 9: Close device
   dwErrCde = DeviceClose();

   return 0;
}
