// ***********************************************************************
// * Program        : timer.c
// * Description    : Demo program for timer function with interrupt
// *
// * Revision       : 1.0
// * Date           : 08/24/2008         Advantech Co., Ltd.
// ***********************************************************************

#include "../include/unodio.h"
#define SUCCESS 0

static pthread_t han_in_id;

void * handle_interrupt(void * arg)
{
   int event_type;
   while(1)
   {
      event_type = CheckEvent( 1 );
      switch( event_type ) 
      {
         case EVT_CNT0:
            printf("Counter0 interrupt occurred!\n");
            break;
         case EVT_CNT1:
            printf("Counter1 interrupt occurred!\n");
            break;
         case EVT_TIMEOUT:
            printf("time out!\n");
            break;
      }
   }
}

int main ( void )
{
   int dwErrCde, fd;
   int usChan;
   int nEventType;
   int ulHardwareCounter;
   int nTimeBase, nTimeBaseHW = TIME_BASE_100;
 	
   //Step 1: Display 
   printf("Timer interrupt with interrupt function testing....\n\n");

   //Step 2: Open device
   fd = DeviceOpen();
   if ( fd == -1 )
   {
      printf("Device Open error!\n");
      return 1;
   }
   
   //Step 3: choose which channel do you want to use
   printf("\nPlease input parameters ---> counter channel:");
   scanf("%d", &usChan);
   
   dwErrCde = CounterReset(usChan);
   if (dwErrCde == -1 && errno == ENOTSUP)
   {
      printf("This funtion do not support\n");
      return 1; 
   }

   switch ( usChan )
   {
      case COUNTER_0:
         nEventType = ADS_EVT_CONT0;
         break;
      case COUNTER_1:
         nEventType = ADS_EVT_CONT1;
         break;
      case COUNTER_32:
         nEventType = ADS_EVT_CONT0 | ADS_EVT_CONT1;
         break;
      default:
         printf( "Counter channel is invalid!\n" );
         return 1;
   }
 
   // Step 4: Config counter (items: time base chosing)
   printf("\nPlease input the time base mode: ");
   printf("\n0 -- TIME BASE is 100K ");
   printf("\n1 -- TIME BASE is 10K  ");
   printf("\n2 -- TIME BASE is 1K   ");
   printf("\n3 -- TIME BASE is 100  \n");
   scanf("%d", &nTimeBase); // make sure the input is from 0 to 3
   if (nTimeBase == 0) 
      nTimeBaseHW = TIME_BASE_100K;
   else if (nTimeBase == 1) 
      nTimeBaseHW = TIME_BASE_10K;
   else if (nTimeBase == 2) 
      nTimeBaseHW = TIME_BASE_1K;
   else if (nTimeBase == 3) 
      nTimeBaseHW = TIME_BASE_100;

   dwErrCde = CounterClockConfig(nTimeBaseHW);	
   if (dwErrCde != 0 )
      return 1;

   // Step 5: enable and start the event function
   if (EventEnable(nEventType, 1, 1))
      return 1;

   // Step 6: Start counter operation by CounterEventStart
   ulHardwareCounter = 0xffff;	// for internal clock = 100K
   dwErrCde = CounterTimerStart(usChan, ulHardwareCounter);
   if (dwErrCde != SUCCESS)
      return 1;

   printf("Press 'q' to exit.\n"); 
	
   dwErrCde = pthread_create( &han_in_id,NULL,handle_interrupt,NULL ); 	
   if ( dwErrCde != EOK )
   {
      printf("create thread failure!\n");
      perror("pthread_create interrrupt:" ); 
         return EXIT_FAILURE;
   } 
   while ( 1 )  
   { 
      if ( getchar() == 'q' )
      {   
         pthread_cancel(han_in_id);
         break;
      } 
      delay( 1000 );
   }
      	
   // Step 7: disable and close the event function
   if (EventEnable(nEventType, 0, 1))
      return (1);

   // Step 8: Stop counter by CounterReset
   dwErrCde = CounterReset(usChan);
   if (dwErrCde != SUCCESS)
      return (1);

   // Step 9: Close device
   dwErrCde = DeviceClose();

   return 0;
}
