/* #############################################################################
   *****************************************************************************                  
               Copyright (c) 2010, Advantech eAutomation Division
                              Advantech Co., Ltd.
                               QNX DIO driver
        THIS IS AN UNPUBLISHED WORK CONTAINING CONFIDENTIAL AND PROPRIETARY
                 INFORMATION WHICH IS THE PROPERTY OF ADVANTECH AUTOMATION CORP.
      ANY DISCLOSURE, USE, OR REPRODUCTION, WITHOUT WRITTEN AUTHORIZATION FROM
                 ADVANTECH AUTOMATION CORP., IS STRICTLY PROHIBITED.
   *****************************************************************************
   #############################################################################
   File:           libads.c
   Version:        1.02 
   Author:         jianfeng
   Description:    UNO DIO QNX driver lib for user application
   Change log:     Version 1.00 <09/11/2008>
                   - Initial version
                     Version 1.01 <03/28/2010>
                   - support UNO-1171, and add function for UNO-1171 program LED
                     Version 1.02 <11/21/2010>
                   - add detecting device function

   Permission to use, copy, modify, and distribute this software and its
   documentation for any purpose and without fee is hereby granted, provided
   that the above copyright notice appear in all copies and that both that
   copyright notice and this permission notice appear in supporting
   documentation.  This software is provided "as is" without express or
   implied warranty.
  ----------------------------------------------------------------------------*/
#include <errno.h>
#include <sys/iofunc.h>
#include <sys/siginfo.h>
#include <time.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <devctl.h>
#include <string.h>

#include <stdio.h>
#include <stdlib.h>

#include <signal.h>
#include "../include/unodioioctl.h"
#include "../include/unodio.h"

int DIChannelNum = 0;
int DOChannelNum = 0;
int bSupportFunc = 0;


int fd = 0;
static int                 channelId; 
static int                 connectionId; 
static int                 irqId = -1; 
static struct sigevent     notifyEvent; 
static struct _pulse       irqPulse; 
static int                 replyId; 
static timer_t             timerId; 
static struct itimerspec   irqTimeout; 


int unodio_type = 0;
#define	PLED_OFFSET              0x14
#define LED_OFFSET               0x10
#define LED_OFFSET1              0x14
enum unodio_type {
	unodio_4channel = 0, // for UNOs ISA DIO, 4 DIO channel device
	unodio_8channel,     // for UNO-2052(E),2176, 8 DIO channel
	unodio_2679,      // UNO-2679, 2 port, 8DIO for port0 and 1 DI for port1
	unodio_1171,        
	unodio_308x,              
	unodio_4672,         // UNO_4672 : 8channel and PLED
} ; 
static int enable_message_send( void )
{
   channelId=ChannelCreate(0); 
   connectionId=ConnectAttach( 0, 0, channelId, _NTO_SIDE_CHANNEL, 0); 
   // Initialize the notification event to deliver a pulse 
   memset( &notifyEvent, 0, sizeof(notifyEvent)); 
   SIGEV_PULSE_INIT( &notifyEvent, 
      connectionId, 
      SIGEV_PULSE_PRIO_INHERIT, 
      IRQ_NOTIFICATION_CODE, 
      EVT_TIMEOUT); 
   
   // Setup a timer in order to detect timeouts 
   //  (10 million nsecs = .01 secs) 
   memset( &irqTimeout, 0, sizeof(irqTimeout)); 
   irqTimeout.it_value.tv_sec = 0; 
   irqTimeout.it_value.tv_nsec = 0; 
   timer_create( CLOCK_REALTIME, &notifyEvent, &timerId); 
   
   // Tell the server to notify you when an IRQ occurs 
   ionotify( fd, 
      _NOTIFY_ACTION_POLLARM, 
      _NOTIFY_COND_INPUT, 
      &notifyEvent); 
   
   // Start the timeout timer 
   timer_settime(timerId, 0, &irqTimeout, NULL); 
}

static int disable_message_send( void )
{
   ConnectDetach( connectionId );
   ChannelDestroy(channelId );
}

int DeviceOpen ( )
{
   int status, ret;
   if((fd = open("/dev/unodio", O_RDWR)) < 0) 
   {
      //printf("cannot open device file /dev/unodio!\n");
      errno = ENODEV;
      return -1 ;
   }
   status = devctl(fd, AD_GETIOTYPE, &unodio_type, sizeof(int), &ret );
   GetDeviceInfor (&DIChannelNum, &DOChannelNum, &bSupportFunc);
   if ( status != EOK)   
   {        
      errno = status;
      return -1;                  
   }
   return fd;
}

int DeviceClose (  )
{
   if(close(fd)) 
   {
      //printf("close device file error!\n");
      errno = EBADF;
      return -1;
   }
   return 0;
}


int GetDeviceInfor(int * DIChannelNum,
	int * DOChannelNum,
	int * bSupportFunc)
{

	//UNO-1171
  	if( unodio_type == unodio_1171 ) 
	{
		*DIChannelNum = 8;
		*DOChannelNum = 8;
		*bSupportFunc =  FUNC_DIO | FUNC_PLED | FUNC_BUZZER ;
	}
	//UNO-2050,UNO-2176
	else if ( unodio_type == unodio_8channel )
	{
		*DIChannelNum = 8;
		*DOChannelNum = 8;
		*bSupportFunc =  FUNC_DIO | FUNC_COUNTER | FUNC_LED | FUNC_BUZZER | FUNC_DIINT;
	}
	//UNO-2679
	else if ( unodio_type == unodio_2679)
	{
		*DIChannelNum = 9;
		*DOChannelNum = 8;
 
		*bSupportFunc =  FUNC_DIO;
	}
	//UNO-3082, UNO-3084
	else if ( unodio_type == unodio_308x)
	{
		*DIChannelNum = 4;
		*DOChannelNum = 4;
		*bSupportFunc =  FUNC_DIO | FUNC_COUNTER | FUNC_POWERSTATUS | FUNC_DIINT;
	}
	//UNO-4672
	else if (unodio_type == unodio_4672)
	{
		*DIChannelNum = 8;
		*DOChannelNum = 8;
		*bSupportFunc =  FUNC_DIO | FUNC_COUNTER | FUNC_LED | FUNC_PLED | FUNC_BUZZER | FUNC_DIINT;
	}
	//UNO-3062:5,UNO-362L:5,UNO-3072:6,UNO-3074:7
	else
	{
		*DIChannelNum = 4;
		*DOChannelNum = 4;
		*bSupportFunc =  FUNC_DIO | FUNC_COUNTER | FUNC_LED | FUNC_BUZZER | FUNC_POWERSTATUS | FUNC_DIINT;
	}
}

int GetPowerStatus ( int * usState )
{
   int status, ret;
   if ((bSupportFunc & FUNC_POWERSTATUS) == 0)
   {
      //printf("This function is not support for the device\n");
      errno = ENOTSUP;
      return -1;
   }
   status = devctl(fd, AD_GETPOWER, usState, sizeof(int), &ret );
   if ( status != EOK)   
   {        
      //there hava errror need handle
      errno = status;
      //printf(" Device IO error!\n");
      return -1;                  //
   }
   if ( ret < 0 )
   {
      //printf(" GetPowerStatus error!\n");
      errno = EIO;
      return -1;
   }
   return 0;
}

int DIReadPortByte(int port, int *value)
{
   int status, ret;
   rsdata_t send_receive_data;
   send_receive_data.i.port = port;

   if ((bSupportFunc & FUNC_DIO) == 0)
   {
      //printf("This function is not support for the device\n");
      errno = ENOTSUP;
      return -1;
   }

   status = devctl(fd, AD_DIO_READ_PORT_BYTE, 
    &send_receive_data, 
     sizeof(send_receive_data),
      &ret );
   if ( status != EOK)   
   {        
      //there hava errror need handle
      errno = status;
      //printf("Device IO error!\n");
      return -1;                  //
   }
   if ( ret < 0 )
   {
      //printf(" DIReadPortByte error!\n");
      errno = EIO;
      return -1;
   }
   *value = send_receive_data.o.status;
   return 0;
}

int DOWritePortByte(int port, int mask, int value)
{
   int status, ret;
   struct rdata_t send_data;
   
   if ((bSupportFunc & FUNC_DIO) == 0)
   {
      //printf("This function is not support for the device\n");
      errno = ENOTSUP;
      return -1;
   }

   send_data.bit = mask;
   send_data.state = value;

   status = devctl(fd, 
      AD_DIO_WRITE_PORT_BYTE, 
      &send_data, 
      sizeof(send_data), 
      &ret );
   if ( status != EOK)   
   {        
      //there hava errror need handle
      errno = status;
      //printf("Device IO error!\n");
      return -1;                 
   }
   if ( ret < 0 )
   {
      //printf("DOWritePortByte error!\n");
      errno = EIO;
      return -1;
   }

   return 0;
}

int DIReadBit(int port, int bit, int *state)
{
   int status, ret;
   rsdata_t send_receive_data;
   
   if ( bit < 0 || bit > 8)
   {
      //printf("invalid DI channel!\n");
      errno = EINVAL;
      return -1;
   }

   if ((bSupportFunc & FUNC_DIO) == 0)
   {
      //printf("This function is not support for the device\n");
      errno = ENOTSUP;
      return -1;
   }

   send_receive_data.i.port = port;
   send_receive_data.i.bit = bit;
   send_receive_data.i.port = port;
   
   status = devctl(fd, 
      AD_DIO_READ_BIT, 
      &send_receive_data, 
      sizeof(send_receive_data), 
      &ret);
   if ( status != EOK)   
   {        
      //there hava errror need handle
      errno = status;
      //printf("Device IO error!\n");
      return -1;                  //
   }
   if ( ret < 0 )
   {
      //printf("DIReadBit error!\n");
      errno = EIO;
      return -1;
   }

   *state = send_receive_data.o.status&0x1;
  // printf("state = %d\n",*state);
   return 0;
}

int DOWriteBit(int port, int bit, int state)
{
   int status, ret;
   struct rdata_t send_data;
   
   if (port != 0 || bit < 0 || bit >8)
   {
      //printf("invalid DO channel!\n");
      errno = EINVAL;
      return -1;
   }

   if ((bSupportFunc & FUNC_DIO) == 0)
   {
      //printf("This function is not support for the device\n");
      errno = ENOTSUP;
      return -1;
   }

   send_data.bit = bit;
   send_data.state = state;
   
   status = devctl(fd, 
      AD_DIO_WRITE_BIT, 
      &send_data, 
      sizeof(send_data), 
      &ret);
   if ( status != EOK)   
   {        
      //there hava errror need handle
      errno = status;
      //printf("Device IO error!\n");
      return -1;  
   }
   if ( ret < 0 )
   {
      //printf("DOWriteBit error!\n");
      errno = EIO;
      return -1;
   }
   return 0;
}

int DOGetCurrentByte(int port, int *value)
{
   int status, ret;

   if ((bSupportFunc & FUNC_DIO) == 0)
   {
      //printf("This function is not support for the device\n");
      errno = ENOTSUP;
      return -1;
   }

   status = devctl(fd, 
      AD_DIO_GET_DO_BYTE, 
      value, 
      sizeof(int), 
      &ret);
   if ( status != EOK)   
   {        
      //there hava errror need handle
      errno = status;
      //printf("Device error!\n");
      return -1;                  
   }   
   if ( ret < 0 )
   {
      //printf("DOGetCurrentByte error!\n");
      errno = EIO;
      return -1;
   }
   return 0;
}

int DOGetCurrentBit(int port, int bit, int *state)
{
   int status, ret;
   rsdata_t send_receive_data;
   
   if (port != 0 || bit < 0 || bit > 8)
   {
      //printf("invalid DO channel!\n");
      errno = EINVAL;
      return -1;
   }

   if ((bSupportFunc & FUNC_DIO) == 0)
   {
      //printf("This function is not support for the device\n");
      errno = ENOTSUP;
      return -1;
   }

   send_receive_data.i.bit = bit;
   //send_data.state = value;
   
   status = devctl(fd, 
      AD_DIO_GET_DO_BIT, 
      &send_receive_data, 
      sizeof(send_receive_data), 
      &ret);
   if ( status != EOK)   
   {        
      //there hava errror need handle
      errno = status;
      //printf("Device error!\n");
      return -1; 
   }
   if ( ret < 0 )
   {
      //printf("DOGetCurrentBit error!\n");
      errno = EIO;
      return -1;
   }
   
   *state = send_receive_data.o.status & 0x1;
   
   return 0;
}   

int CounterTimerStart(int counter, int initHWCount)
{
   int status, ret;
   struct rdata_t send_data;
   
   if ((bSupportFunc & FUNC_COUNTER) == 0)
   {
      //printf("This function is not support for the device\n");
      errno = ENOTSUP;
      return -1;
   }

   send_data.port = counter;
   send_data.state = initHWCount;
   
   status = devctl(fd, 
      AD_COUNTER_TIMER_START, 
      &send_data, 
      sizeof(send_data), 
      &ret);
   if ( status != EOK)   
   {        
      //there hava errror need handle
      errno = status;
      //printf("Device error!\n");
      return -1;                  //
   }
   if ( ret < 0 )
   {
      //printf("CounterTimerStart error!\n");
      errno = EIO;
      return -1;
   }
           
   return 0;
}

int CounterEventStart(int counter, int usCTRGate)
{
   int status, ret;
   struct rdata_t send_data;

   if ((bSupportFunc & FUNC_COUNTER) == 0)
   {
      //printf("This function is not support for the device\n");
      errno = ENOTSUP;
      return -1;
   }

   send_data.port = counter;
   send_data.state = usCTRGate;
   
   status = devctl(fd, 
      AD_COUNTER_EVENT_START, 
      &send_data, 
      sizeof(send_data), 
      &ret);
   if ( status != EOK)   
   {        
      //there hava errror need handle
      errno = status;
      //printf("Device error!\n");
      return -1;  
   }
   if ( ret < 0 )
   {
      //printf("CounterEventStart error!\n");
      errno = EIO;
      return -1;
   }
           
   return 0;
}

int CounterEventRead(int counter, int *overflow, int *count)
{
   int status, ret;
   rsdata_t send_receive_data;

   if ((bSupportFunc & FUNC_COUNTER) == 0)
   {
      //printf("This function is not support for the device\n");
      errno = ENOTSUP;
      return -1;
   }
   
   send_receive_data.i.port = counter;
   
   status = devctl(fd, 
      AD_COUNTER_EVENT_READ, 
      &send_receive_data, 
      sizeof(send_receive_data), 
      &ret);
   if ( status != EOK)   
   {        
      //there hava errror need handle
      errno = status;
      //printf("Device IO error!\n");
      return -1;           
   }
   if ( ret < 0 )
   {
      //printf("CounterEventRead error!\n");
      errno = EIO;
      return -1;
   }

   *overflow = send_receive_data.o.overflow_data;
   *count = send_receive_data.o.status;
   
   return 0;
}

int CounterFreqStart(int counter,  int usCTRGate)
{
   int status, ret;
   struct rdata_t send_data;

   if ((bSupportFunc & FUNC_COUNTER) == 0)
   {
      //printf("This function is not support for the device\n");
      errno = ENOTSUP;
      return -1;
   }

   send_data.port = counter;
   //send_data.bit = GatePeriod;
   send_data.state = usCTRGate;
   
   status = devctl(fd, 
      AD_COUNTER_FREQ_START, 
      &send_data, 
      sizeof(send_data), 
      &ret );
   if ( status != EOK)   
   {        
      //there hava errror need handle
      errno = status;
      //printf("Device IO error!\n");
      return -1;  
   }
   if ( ret < 0 )
   {
      //printf("CounterFreqStart error!\n");
      errno = EIO;
      return -1;
   }
   
   return 0;
}

int CounterFreqRead ( int counter, float *freq )
{
   int status, ret;
   counter_rsdata_t send_receive_data;

   if ((bSupportFunc & FUNC_COUNTER) == 0)
   {
      //printf("This function is not support for the device\n");
      errno = ENOTSUP;
      return -1;
   }
   
   send_receive_data.i.counter = counter;
   
   status = devctl(fd, 
      AD_COUNTER_FREQ_READ, 
      &send_receive_data, 
      sizeof(send_receive_data), 
      &ret);
   if ( status != EOK)   
   {        
      //there hava errror need handle
      errno = status;
      //printf("Device IO error!\n");
      return -1; 
   }
   if ( ret < 0 )
   {
      //printf("CounterFreqRead error!\n");
      errno = EIO;
      return -1;
   }
   
   //*freq = data[1];
   *freq = (float)send_receive_data.o.cntnum * (float)send_receive_data.o. kernelhz/ (float)send_receive_data.o.timelapse;   
  /* printf("CNT is %ld, kernelHz is %d, Time is %ld\n", 
      send_receive_data.o.cntnum, 
      send_receive_data.o.kernelhz, 
      send_receive_data.o.timelapse);*/
   return 0;
}

int CounterPulseStart ( 
   int counter, 
   float period, 
   int usCTRGate )
{
   int status, ret;
   //float data[4];
   struct counter_rdata_t send_data;
   send_data.counter = counter;
   send_data.period = period;
   //send_data.UpCycle = upCycle;
   send_data.usCTRGate = usCTRGate;
   
   if ((bSupportFunc & FUNC_COUNTER) == 0)
   {
      //printf("This function is not support for the device\n");
      errno = ENOTSUP;
      return -1;
   }

   status = devctl(fd, 
      AD_COUNTER_PULSE_START, 
      &send_data, 
      sizeof(send_data), 
      &ret);
   if ( status != EOK)   
   {        
      //there hava errror need handle
      errno = status;
      //printf("Device IO error!\n");
      return -1;          
   }
   if ( ret < 0 )
   {
      //printf("CounterPulseStart error!\n");
      errno = EIO;
      return -1;
   }
   return 0;
}

int CounterReset(int counter)
{
   int status, ret;
      
   if ((bSupportFunc & FUNC_COUNTER) == 0)
   {
      //printf("This function is not support for the device\n");
      errno = ENOTSUP;
      return -1;
   }

   status = devctl(fd, 
      AD_COUNTER_RESET, 
      &counter, 
      sizeof(counter), 
      &ret);
   if ( status != EOK)   
   {        
      //there hava errror need handle
      errno = status;
      //printf("Device IO error!\n");
      return -1;
   }
   if ( ret < 0 )
   {
      //printf("CounterReset error!\n");
      errno = EIO;
      return -1;
   }
   
   return 0;
}

int CounterClockConfig(int timeBase)
{
   int status, ret;

   if ((bSupportFunc & FUNC_COUNTER) == 0)
   {
      //printf("This function is not support for the device\n");
      errno = ENOTSUP;
      return -1;
   }

   status = devctl(fd, 
      AD_COUNTER_CLOCK_CFG, 
      &timeBase, 
      sizeof(timeBase), 
      &ret);
   if ( status != EOK)   
   {       
      //there hava errror need handle
      errno = status;
      //printf("Device IO error!\n");
      return -1;  
   }
   if ( ret < 0 )
   {
      //printf("CounterClockConfig error!\n");
      errno = EIO;
      return -1;
   }

   return 0;
}

int CounterConfig(int counter, int clockPin, int gatePin, int outPin)
{
   int status;
   int ret;

   struct counter_sdata_t send_data;
   if ((bSupportFunc & FUNC_COUNTER) == 0)
   {
      //printf("This function is not support for the device\n");
      errno = ENOTSUP;
      return -1;
   }

   send_data.counter = counter;
   send_data.timelapse = clockPin;
   send_data.cntnum = gatePin;
   send_data.kernelhz = outPin;
   
   status = devctl(fd, 
      AD_COUNTER_CFG, 
      &send_data, 
      sizeof(send_data), 
      &ret);
   if ( status != EOK)   
   {        
      //there hava errror need handle
      errno = status;
      //printf("Device IO error!\n");
      return -1;     
   }
   if ( ret < 0 )
   {
      //printf("CounterConfig error!\n");
      errno = EIO;
      return -1;  
   }
   return 0;   
}

int CounterGateSet( int counter, int CTRGate)
{
   int status, ret;
   struct rdata_t send_data;
   if ((bSupportFunc & FUNC_COUNTER) == 0)
   {
      //printf("This function is not support for the device\n");
      errno = ENOTSUP;
      return -1;
   }

   send_data.port = counter;
   send_data.bit = CTRGate;
   
   status = devctl(fd, AD_COUNTER_GATE_SET, &send_data, sizeof(send_data), &ret);
   if ( status != EOK)   
   {        
      //there hava errror need handle
      errno = status;
      //printf("Device IO error!\n");
      return -1; 
   }
   if ( ret < 0 )
   {
      //printf("CounterGateSet error!\n");
      errno = EIO;
      return -1;
   }
   
   return 0;
}

int WritePortByte(int offset, int value)
{
   int status, ret;
   struct rdata_t send_data;
   send_data.port = offset;
   send_data.state = value;
   
   status = devctl(fd, 
      AD_WRITE_PORT_BYTE, 
      &send_data, 
      sizeof(send_data), 
      &ret);
   if ( status != EOK)   
   {        //there hava errror need handle
      errno = status;
      //printf("Device IO error!\n");
      return -1;    
   }
   if ( ret < 0 )
   {
      //printf("WritePortByte error!\n");
      errno = EIO;
      return -1;
   }

   return 0;   
}

int ReadPortByte(int offset, int* value)
{
   int status, ret;
   rsdata_t send_receive_data;
   send_receive_data.i.port = offset;
   
   status = devctl(fd, 
      AD_READ_PORT_BYTE, 
      &send_receive_data, 
      sizeof(send_receive_data), 
      &ret);
   if ( status != EOK)   
   {        
      //there hava errror need handle
      errno = status;
      //printf("Device IO error!\n");
      return -1;     
   }
   if ( ret < 0 )
   {
      //printf("ReadPortByte error!\n");
      errno = EIO;
      return -1;
   }
   
   *value = send_receive_data.o.status;
   
   return 0;   
}

int EventConfig(int eventType, int triggerEdge)
{
   int status, ret;
   struct rdata_t send_data;
   send_data.port = eventType;
   send_data.state = triggerEdge;
   
   if  (unodio_type == unodio_2679)
   {
      //printf("This function is not support for the device\n");
      errno = ENOTSUP;
      return -1;
   }
   status = devctl(fd, AD_EVENT_CFG, &send_data, sizeof(send_data), &ret);
   if ( status != EOK)   
   {        
      //there hava errror need handle
      errno = status;
      //printf("Device IO error!\n");
      return -1;     
   }
   if ( ret < 0 )
   {
      //printf("EventConfig error!\n");
      errno = EIO;
      return -1;
   }
   return 0;      
}

int EventEnable(int eventType, int enabled, int count)
{
   int status, ret;
   struct rdata_t send_data;
   send_data.port = eventType;
   send_data.bit = enabled;
   send_data.state = count;
   
   status = devctl(fd, AD_EVENT_ENABLE, &send_data, sizeof(send_data), &ret);
   if ( status != EOK)   
   {        
      //there hava errror need handle
      errno = status;
      //printf("Device IO error!\n");
      return -1;                  //
   }
   if ( ret < 0 )
   {
      //printf("EventEnable error!\n");
      errno = EIO;
      return -1;
   }

   if ( enabled == 1)
      enable_message_send();
   else  if( ( channelId != -1 )&&( connectionId != -1 ))
      disable_message_send();
   
   return 0;      
}
int ProgramLedRead(int *value)
{
   int status, ret;

   if ((bSupportFunc & FUNC_PLED) == 0)
   {
      //printf("This function is not support for the device\n");
      errno = ENOTSUP;
      return -1;
   }

   status = devctl(fd, AD_PROGRAMLED_READ, value, sizeof(int), &ret );
   if ( status != EOK)   
   {        
      //there hava errror need handle
      errno = status;
      //printf("Device IO error!\n");
      return -1;                  //
   }
   if ( ret < 0 )
   {
      //printf(" ProgramLedRead error!\n");
      errno = EIO;
      return -1;
   }
   return 0;
}

int ProgramLedWrite(int Mode,int Mask)
{
   int status, ret;
   struct rdata_t send_data;
   
   if ((bSupportFunc & FUNC_PLED) == 0)
   {
      //printf("This function is not support for the device\n");
      errno = ENOTSUP;
      return -1;
   }

   send_data.bit = Mask;
   send_data.state = Mode;
   
   status = devctl(fd, 
      AD_PROGRAMLED_WRITE, 
      &send_data, 
      sizeof(send_data), 
      &ret );
   if ( status != EOK)   
   {        
      //there hava errror need handle
      errno = status;
      //printf("Device IO error!\n");
      return -1;                 
   }
   if ( ret < 0 )
   {
      //printf("LEDToggle error\n");
      errno = EIO;
      return -1;
   }
}


// *****************************************************************************
// Design Notes:  
// -----------------------------------------------------------------------------
int ProgramLedModeEnable( int ledChannel, int Enabled )
{

	int  usReadBack;
	int  usData;
	
        if ((bSupportFunc & FUNC_PLED) == 0)
        {
            //printf("This function is not support for the device\n");
            errno = ENOTSUP;
            return -1;
        }	

	ReadPortByte( PLED_OFFSET+1 , &usReadBack);

	if (Enabled == 1)
	{
		usData = ( 1 << ledChannel ) | usReadBack ;
	}
	else
	{
		usData = ~( 1 << ledChannel ) & usReadBack;
	}
	WritePortByte( PLED_OFFSET+1, usData & 0xF );
	
	return 0;       
}

// *****************************************************************************
// Design Notes:  
// -----------------------------------------------------------------------------
int ProgramLedModeConfig( int usMode )
{
   if ((bSupportFunc & FUNC_PLED) == 0)
   {
      //printf("This function is not support for the device\n");
      errno = ENOTSUP;
      return -1;
   }
   // Mode Description
   // LED_FAST     0x3  // fast flicker
   // LED_NORMAL   0x5  // normal flick
   // LED_SLOW     0x7  // slow flicker    

   WritePortByte( LED_OFFSET, usMode | 0x1 );

   return 0;       
}

int LedEnable(int mode)
{
   int status, ret;
   if ((bSupportFunc & FUNC_LED) == 0)
   {
      //printf("This function is not support for the device\n");
      errno = ENOTSUP;
      return -1;
   }
   status = devctl(fd, AD_LED_ENABLE, &mode, sizeof(mode), &ret);
   if ( status != EOK)   
   {        
      //there hava errror need handle
      errno = status;
      //printf("Device IO error!\n");
      return -1;             
   }
   if ( ret < 0 )
   {
      //printf("LedEnable error!\n");
      errno = EIO;
      return -1;
   }
        
   return 0;   
}

int BuzzerEnable(int mode)
{
   int status, ret;
   if ((bSupportFunc & FUNC_BUZZER) == 0)
   {
      //printf("This function is not support for the device\n");
      errno = ENOTSUP;
      return -1;
   }
   status = devctl(fd, AD_BUZZER_ENABLE, &mode, sizeof(mode), &ret);
   if ( status != EOK)   
   {        
      //there hava errror need handle
      errno = status;
      //printf("Device IO error!\n");
      return -1;     
   }
   if ( ret < 0 )
   {
      //printf("BuzzerEnable error!\n");
      errno = EIO;
      return -1;
   }
   
   return 0;   
}

/*
int getcounter( long * isr_counter, long * dpc_counter)
{    
      //printf("come get counter\n");
      devctl(fd, AD_GETISRCOUNTER, isr_counter, sizeof(long), NULL );
      devctl(fd, AD_GETDPCCOUNTER,dpc_counter, sizeof(long), NULL );
      return 0;
}
*/

// check interrupt function
int CheckEvent(int timeout)
{
   irqTimeout.it_value.tv_sec = timeout; 
   irqTimeout.it_value.tv_nsec = 0; 
   timer_settime(timerId, 0, &irqTimeout, NULL);
   irqId = -1;
   // Loop until you get an IRQ pulse, 
   // a timeout pulse or something bad happens 
   while(   irqId == -1 ) 
   { 
      // Wait for a pulse to arrive 
      replyId = MsgReceivePulse( channelId, 
         &irqPulse, 
         sizeof( irqPulse), 
         NULL); 
      if( replyId == -1) 
      { 
         // an error occurred 
         irqId = -1; 
         perror(NULL);
      } 
      else if( replyId == 0) 
      { 
         // You got a legitimate pulse..hanlde it 
         // Check if this is an IRQ notification pulse 
         if( irqPulse.code == IRQ_NOTIFICATION_CODE) 
         { 
            // Disarm the timeout timer 
            irqTimeout.it_value.tv_sec = 0;
            irqTimeout.it_value.tv_nsec = 0; 
            timer_settime(timerId, 0, &irqTimeout, NULL); 

            // See which event the driver is notifying you about 
            //printf(" irqPulse.value.sival_int is %d\n", irqPulse.value.sival_int);
            irqId = irqPulse.value.sival_int ;
	    /*
            switch( irqPulse.value.sival_int  ) 
            {
               case  EVT_DI0:
                  irqId  = EVT_DI0;
                  break;
               case  EVT_DI1:
                  irqId  = EVT_DI1;
                  break;              
               case  EVT_DI:
                  irqId  = EVT_DI2;
                  break;
               case  EVT_DI:
                  irqId  = EVT_DI3;
                  break;              
               case  EVT_DI:
                  irqId  = EVT_DI4;
                  break;
               case  EVT_DI5:
                  irqId  = EVT_DI5;
                  break;              
               case  EVT_DI6:
                  irqId  = EVT_DI6;
                  break;
               case  EVT_DI7:
                  irqId  = EVT_DI7;
                  break;              
               case  EVT_DI8:
                  irqId  = EVT_DI8;
                  break;
               case EVT_CNT0;	   
                  irqId = EVT_CNT0;
                  break;
               case EVT_CNT1:
                  irqId = EVT_CNT1;
                  break;
               case EVT_TIMEOUT: 
                  irqId = irqPulse.value.sival_int; 
                  break; 
               default: 
                  irqId = -1; 
                  break; 
            }            
            */ 
         } 
      } 
   } 
   // ConnectDetach( connectionId );
   // ChannelDestroy(channelId );
   return( irqId); 
}

